@@ -1,6 +1,6 @@
 ###############################################################################
 #
-# This file copyright (c) 2001-2010 Randy J. Ray, all rights reserved
+# This file copyright (c) 2001-2011 Randy J. Ray, all rights reserved
 #
 # Copying and distribution are permitted under the terms of the Artistic
 # License 2.0 (http://www.opensource.org/licenses/artistic-license-2.0.php) or
@@ -22,12 +22,13 @@
 
 package RPC::XML;
 
-use 5.006001;
+use 5.008008;
 use strict;
 use warnings;
 use vars qw(@EXPORT_OK %EXPORT_TAGS $VERSION $ERROR
-            %XMLMAP $XMLRE $ENCODING $FORCE_STRING_ENCODING $ALLOW_NIL);
-use subs qw(time2iso8601 smart_encode utf8_downgrade);
+            %XMLMAP $XMLRE $ENCODING $FORCE_STRING_ENCODING $ALLOW_NIL
+            $DATETIME_REGEXP $DATETIME_ISO8601_AVAILABLE);
+use subs qw(time2iso8601 smart_encode);
 use base 'Exporter';
 
 use Scalar::Util qw(blessed reftype);
@@ -47,21 +48,22 @@ BEGIN
     # Allow the <nil /> extension?
     $ALLOW_NIL = 0;
 
-    # Cribbed from the UTF-8 fixes in HTTP::Message, this may be discardable
-    # once full encoding support is in place:
-    *utf8_downgrade = defined(&utf8::downgrade) ?
-        \&utf8::downgrade : sub { };
+    # Determine if the DateTime::Format::ISO8601 module is available for
+    # RPC::XML::datetime_iso8601 to use:
+    my $retval = eval 'use DateTime::Format::ISO8601; 1;';
+    $DATETIME_ISO8601_AVAILABLE = $retval ? 1 : 0;
 }
 
 @EXPORT_OK = qw(time2iso8601 smart_encode
-                RPC_BOOLEAN RPC_INT RPC_I4 RPC_DOUBLE RPC_DATETIME_ISO8601
-                RPC_BASE64 RPC_STRING RPC_NIL
+                RPC_BOOLEAN RPC_INT RPC_I4 RPC_I8 RPC_DOUBLE
+                RPC_DATETIME_ISO8601 RPC_BASE64 RPC_STRING RPC_NIL
                 $ENCODING $FORCE_STRING_ENCODING $ALLOW_NIL);
-%EXPORT_TAGS = (types => [ qw(RPC_BOOLEAN RPC_INT RPC_I4 RPC_DOUBLE RPC_STRING
-                              RPC_DATETIME_ISO8601 RPC_BASE64 RPC_NIL) ],
+%EXPORT_TAGS = (types => [ qw(RPC_BOOLEAN RPC_INT RPC_I4 RPC_I8 RPC_DOUBLE
+                              RPC_STRING RPC_DATETIME_ISO8601 RPC_BASE64
+                              RPC_NIL) ],
                 all   => [ @EXPORT_OK ]);
 
-$VERSION = '1.52';
+$VERSION = '1.56';
 $VERSION = eval $VERSION; ## no critic (ProhibitStringyEval)
 
 # Global error string
@@ -78,6 +80,25 @@ $ERROR = q{};
 );
 $XMLRE = join q{} => keys %XMLMAP; $XMLRE = qr/([$XMLRE])/;
 
+# The XMLRPC spec only allows for the incorrect iso8601 format
+# without dashes, but dashes are part of the standard so we include
+# them. Note that the actual RPC::XML::datetime_iso8601 class will strip
+# them out if present.
+my $date_re =
+    qr{
+          (\d{4})-?
+          ([01]\d)-?
+          ([0123]\d)
+    }x;
+my $time_re =
+    qr{
+          ([012]\d):
+          ([0-5]\d):
+          ([0-5]\d)([.,]\d+)?
+          (Z|[-+]\d\d:\d\d)?
+    }x;
+$DATETIME_REGEXP = qr{^${date_re}T?${time_re}$};
+
 # All of the RPC_* functions are convenience-encoders
 sub RPC_STRING ($)
 {
@@ -122,17 +143,10 @@ sub RPC_NIL ()
 sub time2iso8601
 {
     my $time = shift || time;
-    my $zone = shift || q{};
 
     my @time = gmtime $time;
     $time = sprintf '%4d%02d%02dT%02d:%02d:%02dZ',
                     $time[5] + 1900, $time[4] + 1, @time[3, 2, 1, 0];
-    if ($zone)
-    {
-        my $char = $zone > 0 ? q{+} : q{-};
-        chop $time; # Lose the Z if we're specifying a zone
-        $time .= $char . sprintf '%02d:00', abs $zone;
-    }
 
     return $time;
 }
@@ -169,7 +183,7 @@ sub time2iso8601
 
         foreach (@values)
         {
-            if (! defined $_) ## no critic (ProhibitCascadingIfElse)
+            if (! defined $_)
             {
                 $type = $ALLOW_NIL ?
                     RPC::XML::nil->new() : RPC::XML::string->new(q{});
@@ -179,7 +193,7 @@ sub time2iso8601
                 # Skip any that we've already seen
                 next if $seenrefs->{$_}++;
 
-                if (blessed($_) && ## no critic (ProhibitCascadingIfElse)
+                if (blessed($_) &&
                     ($_->isa('RPC::XML::datatype') || $_->isa('DateTime')))
                 {
                     # Only if the reference is a datatype or a DateTime
@@ -194,7 +208,7 @@ sub time2iso8601
                     {
                         # Must be a DateTime object, convert to ISO8601
                         $type = RPC::XML::datetime_iso8601
-                            ->new($_->clone->set_time_zone('UTC')->iso8601);
+                            ->new($_->clone->set_time_zone('UTC'));
                     }
                 }
                 elsif (reftype($_) eq 'HASH')
@@ -263,31 +277,13 @@ sub time2iso8601
             }
             # Pattern taken from perldata(1)
             elsif (! $FORCE_STRING_ENCODING &&
-                   /^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/x &&
+                   /^[+-]?(?=\d|[.]\d)\d*(?:[.]\d*)?(?:[Ee](?:[+-]?\d+))?$/x &&
                    $_ > $MIN_DOUBLE &&
                    $_ < $MAX_DOUBLE)
             {
                 $type = RPC::XML::double->new($_);
             }
-            # The XMLRPC spec only allows for the incorrect iso8601 format
-            # without dashes, but dashes are part of the standard so we include
-            # them (DateTime->now->iso8601 includes them). Note that the actual
-            # RPC::XML::datetime_iso8601 class will strip them out if present.
-            elsif (m{
-                       ^           # start
-                       \d{4}       # 4 digit year
-                       -?          # "optional" dash
-                       [01]\d      # month
-                       -?          # optional dash
-                       [0123]\d    # day of month
-                       T           # Yes, "T"
-                       [012]\d:    # hours
-                       [012345]\d: # min
-                       [012345]\d  # seconds
-                       (?:\.\d+)?  # optional fractional seconds
-                       Z?          # optional "Z" to indicate UTC
-                       $           # end
-                }x)
+            elsif (/$DATETIME_REGEXP/)
             {
                 $type = RPC::XML::datetime_iso8601->new($_);
             }
@@ -342,6 +338,14 @@ sub new
 
     $RPC::XML::ERROR = q{};
     $class = ref($class) || $class;
+
+    if ($class eq 'RPC::XML::simple_type')
+    {
+        $RPC::XML::ERROR = 'RPC::XML::simple_type::new: Cannot instantiate ' .
+            'this class directly';
+        return;
+    }
+
     if (ref $value)
     {
         # If it is a scalar reference, just deref
@@ -354,6 +358,7 @@ sub new
             # We can only manage scalar references (or blessed scalar refs)
             $RPC::XML::ERROR = "${class}::new: Cannot instantiate from a " .
                 'reference not derived from scalar';
+            return;
         }
     }
 
@@ -365,6 +370,13 @@ sub value
 {
     my $self = shift;
 
+    if (! ref $self)
+    {
+        $RPC::XML::ERROR =
+            "{$self}::value: Cannot be called as a static method";
+        return;
+    }
+
     return ${$self};
 }
 
@@ -376,6 +388,8 @@ sub as_string
     my $class = ref $self;
     if (! $class)
     {
+        $RPC::XML::ERROR =
+            "{$self}::as_string: Cannot be called as a static method";
         return;
     }
     $class =~ s/^.*\://;
@@ -393,10 +407,9 @@ sub serialize
 {
     my ($self, $fh) = @_;
 
-    my $str = $self->as_string;
-    RPC::XML::utf8_downgrade($str);
-
+    utf8::downgrade(my $str = $self->as_string);
     print {$fh} $str;
+
     return;
 }
 
@@ -406,7 +419,7 @@ sub length ## no critic (ProhibitBuiltinHomonyms)
 {
     my $self = shift;
 
-    RPC::XML::utf8_downgrade(my $str = $self->as_string);
+    utf8::downgrade(my $str = $self->as_string);
 
     return length $str;
 }
@@ -463,13 +476,15 @@ sub as_string
 {
     my $self = shift;
 
-    my $class = $self->type;
-    if (! $class)
+    if (! ref $self)
     {
+        $RPC::XML::ERROR =
+            "{$self}::as_string: Cannot be called as a static method";
         return;
     }
+    my $class = $self->type;
 
-    (my $value = sprintf '%.20f', ${$self}) =~ s/(\.\d+?)0+$/$1/;
+    (my $value = sprintf '%.20f', ${$self}) =~ s/([.]\d+?)0+$/$1/;
 
     return "<$class>$value</$class>";
 }
@@ -493,11 +508,13 @@ sub as_string
 
     my ($class, $value);
 
-    $class = $self->type;
-    if (! $class)
+    if (! ref $self)
     {
+        $RPC::XML::ERROR =
+            "{$self}::as_string: Cannot be called as a static method";
         return;
     }
+    $class = $self->type;
 
     ($value = defined ${$self} ? ${$self} : q{} )
         =~ s/$RPC::XML::XMLRE/$RPC::XML::XMLMAP{$1}/ge;
@@ -563,35 +580,52 @@ sub type { return 'dateTime.iso8601'; };
 sub new
 {
     my ($class, $value) = @_;
+    my $newvalue;
 
     if (ref($value) && reftype($value) eq 'SCALAR')
     {
         $value = ${$value};
     }
 
-    if ($value && $value =~ m{^(\d{4})-?([01]\d)-?([0123]\d)T
-                              ([012]\d):([012345]\d):([012345]\d)(\.\d+)?
-                              (Z|[-+]\d\d:\d\d)?$}x)
+    if (defined $value)
     {
-        # This is the WRONG way to represent this, but it's the way it is
-        # given in the spec, so assume that other implementations can only
-        # accept this form. Also, this should match the form that time2iso8601
-        # produces.
-        $value = $7 ? "$1$2$3T$4:$5:$6$7" : "$1$2$3T$4:$5:$6";
-        if ($8)
+        if ($value =~ /$RPC::XML::DATETIME_REGEXP/)
         {
-            $value .= $8;
+            # This is *not* a valid ISO 8601 format, but it's the way it is
+            # given in the spec, so assume that other implementations can only
+            # accept this form. Also, this should match the form that
+            # time2iso8601 produces.
+            $newvalue = $7 ? "$1$2$3T$4:$5:$6$7" : "$1$2$3T$4:$5:$6";
+            if ($8) {
+                $newvalue .= $8;
+            }
+        }
+        elsif ($RPC::XML::DATETIME_ISO8601_AVAILABLE)
+        {
+            $newvalue =
+                eval { DateTime::Format::ISO8601->parse_datetime($value) };
+            if ($newvalue)
+            {
+                # This both removes the dashes (*sigh*) and forces it from an
+                # object to an ordinary string:
+                $newvalue =~ s/-//g;
+            }
+        }
+
+        if (! $newvalue)
+        {
+            $RPC::XML::ERROR = "${class}::new: Malformed data ($value) " .
+                'passed as dateTime.iso8601';
+            return;
         }
     }
     else
     {
-        $RPC::XML::ERROR = "${class}::new: Malformed data (" .
-            (defined($value) ? $value : '<undef>') .
-            ') passed as dateTime.iso8601';
+        $RPC::XML::ERROR = "${class}::new: Value required in constructor";
         return;
     }
 
-    return bless \$value, $class;
+    return bless \$newvalue, $class;
 }
 
 ###############################################################################
@@ -823,7 +857,7 @@ sub serialize
     for (keys %{$self})
     {
         ($key = $_) =~ s/$RPC::XML::XMLRE/$RPC::XML::XMLMAP{$1}/ge;
-        RPC::XML::utf8_downgrade($key);
+        utf8::downgrade($key);
         print {$fh} "<member><name>$key</name><value>";
         $self->{$_}->serialize($fh);
         print {$fh} '</value></member>';
@@ -843,7 +877,7 @@ sub length ## no critic (ProhibitBuiltinHomonyms)
     {
         $len += 45; # For all the constant XML presence
         $len += $self->{$key}->length;
-        RPC::XML::utf8_downgrade($key);
+        utf8::downgrade($key);
         $len += length $key;
     }
 
@@ -1082,17 +1116,23 @@ sub to_file
 
     my ($fh, $buf, $do_close, $count) = (undef, q{}, 0, 0);
 
-    if (ref $file and reftype($file) eq 'GLOB')
+    if (ref $file)
     {
-        $fh = $file;
+        if (reftype($file) eq 'GLOB')
+        {
+            $fh = $file;
+        }
+        else
+        {
+            $RPC::XML::ERROR = 'Unusable reference type passed to to_file';
+            return -1;
+        }
     }
     else
     {
-        require Symbol;
-        $fh = Symbol::gensym();
         if (! open $fh, '>', $file) ## no critic (RequireBriefOpen)
         {
-            $RPC::XML::ERROR = $!;
+            $RPC::XML::ERROR = "Error opening $file for writing: $!";
             return -1;
         }
         binmode $fh;
@@ -1129,6 +1169,8 @@ sub to_file
         }
         else
         {
+            # If the data is already decoded in the filehandle, then just copy
+            # it over.
             my $size;
             while ($size = read $self->{value_fh}, $buf, 4096)
             {
@@ -1136,6 +1178,8 @@ sub to_file
                 $count += $size;
             }
         }
+
+        # Restore the position of the file-pointer for the internal FH
         seek $self->{value_fh}, $self->{fh_pos}, 0;
     }
 
@@ -1143,7 +1187,7 @@ sub to_file
     {
         if (! close $fh)
         {
-            $RPC::XML::ERROR = $!;
+            $RPC::XML::ERROR = "Error closing $file after writing: $!";
             return -1;
         }
     }
@@ -1184,7 +1228,7 @@ sub new
         # Take the keys and values from the struct object as our own
         %args = %{$args[0]->value('shallow')};
     }
-    elsif (@args == 2)
+    elsif ((@args == 2) && ($args[0] =~ /^-?\d+$/) && length $args[1])
     {
         # This is a special convenience-case to make simple new() calls clearer
         %args = (faultCode   => RPC::XML::int->new($args[0]),
@@ -1304,6 +1348,14 @@ sub new
 
     # This is the method name to be called
     $name = shift @argz;
+    # Is it valid?
+    if ($name !~ m{^[\w.:/]+$})
+    {
+        $RPC::XML::ERROR =
+            'RPC::XML::request::new: Invalid method name specified';
+        return;
+    }
+
     # All the remaining args must be data.
     @argz = RPC::XML::smart_encode(@argz);
 
@@ -1312,7 +1364,7 @@ sub new
 
 # Accessor methods
 sub name { return shift->{name}; }
-sub args { return shift->{args} || []; }
+sub args { return shift->{args}; }
 
 ###############################################################################
 #
@@ -1356,8 +1408,7 @@ sub as_string
 sub serialize
 {
     my ($self, $fh) = @_;
-    my $name = $self->{name};
-    RPC::XML::utf8_downgrade($name);
+    utf8::downgrade(my $name = $self->{name});
 
     print {$fh} qq(<?xml version="1.0" encoding="$RPC::XML::ENCODING"?>);
 
@@ -1379,7 +1430,8 @@ sub length ## no critic (ProhibitBuiltinHomonyms)
     my $self = shift;
 
     my $len = 100 + length $RPC::XML::ENCODING; # All the constant XML present
-    $len += length $self->{name};
+    utf8::downgrade(my $name = $self->{name});
+    $len += length $name;
 
     for (@{$self->{args}})
     {
@@ -1550,7 +1602,7 @@ RPC::XML - A set of classes for core data, message and XML handling
     use RPC::XML;
 
     $req = RPC::XML::request->new('fetch_prime_factors',
-                                  RPC::XML::int->new(985120528));
+                                  RPC::XML::int->new(985_120_528));
     ...
     $resp = RPC::XML::ParserFactory->new()->parse(STREAM);
     if (ref($resp))
@@ -1564,33 +1616,37 @@ RPC::XML - A set of classes for core data, message and XML handling
 
 =head1 DESCRIPTION
 
-The B<RPC::XML> package is an implementation of the B<XML-RPC> standard.
+The B<RPC::XML> package is an implementation of the B<XML-RPC> standard. The
+package as a whole provides classes for data, for clients, for servers and for
+parsers (based on the L<XML::Parser|XML::Parser> and L<XML::LibXML|XML::LibXML>
+packages from CPAN).
 
-The package provides a set of classes for creating values to pass to the
-constructors for requests and responses. These are lightweight objects, most
-of which are implemented as tied scalars so as to associate specific type
-information with the value. Classes are also provided for requests, responses,
-faults (errors) and a parser based on the L<XML::Parser> package from CPAN.
+This module provides a set of classes for creating values to pass to the
+constructors for requests and responses. These are lightweight objects, most of
+which are implemented as blessed scalar references so as to associate specific
+type information with the value. Classes are also provided for requests,
+responses and faults (errors).
 
-This module does not actually provide any transport implementation or
-server basis. For these, see L<RPC::XML::Client> and L<RPC::XML::Server>,
-respectively.
+This module does not actually provide any transport implementation or server
+basis. For these, see L<RPC::XML::Client|RPC::XML::Client> and
+L<RPC::XML::Server|RPC::XML::Server>, respectively.
 
 =head1 SUBROUTINES/METHODS
 
-At present, three simple subroutines are available for import. They must be
-explicitly imported as part of the C<use> statement, or with a direct call to
-C<import>:
+At present, two subroutines are available for import. They must be explicitly
+imported as part of the C<use> statement, or with a direct call to C<import>:
 
 =over 4
 
 =item time2iso8601([$time])
 
 Convert the integer time value in C<$time> (which defaults to calling the
-built-in C<time> if not present) to a ISO 8601 string in the UTC time
-zone. This is a convenience function for occassions when the return value
-needs to be of the B<dateTime.iso8601> type, but the value on hand is the
-return from the C<time> built-in.
+built-in C<time> if not present) to a (pseudo) ISO 8601 string in the UTC time
+zone. This is a convenience function for occassions when the return value needs
+to be of the B<dateTime.iso8601> type, but the value on hand is the return from
+the C<time> built-in. Note that the format of this string is not strictly
+compliant with ISO 8601 due to the way the B<dateTime.iso8601> data-type was
+defined in the specification. See L</"DATES AND TIMES">, below.
 
 =item smart_encode(@args)
 
@@ -1621,7 +1677,7 @@ B<RPC::XML::datetime_iso8601> class.
 =back
 
 In addition to these, the following "helper" functions are also available. They
-may be imported explicitly, or via a tag of C<:types>:
+may be imported explicitly, or all may be imported via the tag C<:types>:
 
     RPC_BOOLEAN RPC_INT RPC_I4 RPC_I8 RPC_DOUBLE
     RPC_DATETIME_ISO8601 RPC_BASE64 RPC_STRING RPC_NIL
@@ -1635,7 +1691,7 @@ the tag C<:all>.
 
 =head1 CLASSES
 
-The classes provided by this module are broken into two groups: I<datatype>
+The classes provided by this module are broken into two groups: I<data>
 classes and I<message> classes.
 
 =head2 Data Classes
@@ -1667,7 +1723,7 @@ Returns the value as a XML-RPC fragment, with the proper tags, etc.
 =item serialize($filehandle)
 
 Send the stringified rendition of the data to the given file handle. This
-allows messages with arbitrarily-large Base-64 data within them to be sent
+allows messages with arbitrarily-large base-64 data within them to be sent
 without having to hold the entire message within process memory.
 
 =item length
@@ -1730,18 +1786,23 @@ program may specify any of: C<0>, C<no>, C<false>, C<1>, C<yes>, C<true>.
 =item RPC::XML::datetime_iso8601
 
 Creates an instance of the XML-RPC C<dateTime.iso8601> type. The specification
-for ISO 8601 may be found elsewhere. No processing is done to the data.
+for ISO 8601 may be found elsewhere. No processing is done to the data. Note
+that the XML-RPC specification actually got the format of an ISO 8601 date
+slightly wrong. Because this is what is in the published spec, this package
+produces dates that match the XML-RPC spec, not the the ISO 8601 spec. However,
+it will I<read> date-strings in proper ISO 8601 format. See L</"DATES AND
+TIMES">, below.
 
 =item RPC::XML::nil
 
 Creates a C<nil> value. The value returned will always be B<undef>. No value
 should be passed when calling the constructor.
 
-Note that nil is an extension to B<XML-RPC>, which is not supported by
-all implementations. B<$RPC::XML::ALLOW_NIL> must be set to a non-false
-value before objects of this type can be constructed. See
-L</GLOBAL VARIABLES>. If B<$RPC::XML::ALLOW_NIL> is set to a false value,
-the parsers will not recognize the C<< <nil /> >> tag at all.
+Note that nil is an extension to B<XML-RPC>, which is not supported by all
+implementations. B<$RPC::XML::ALLOW_NIL> must be set to a non-false value
+before objects of this type can be constructed. See L</GLOBAL
+VARIABLES>. However, even if B<$RPC::XML::ALLOW_NIL> is set to a false value,
+the parsers will recognize the C<< <nil /> >> tag and construct an object.
 
 In practice, this type is only useful to denote the equivalent of a "void"
 return value from a function. The type itself is not interchangeable with
@@ -1911,10 +1972,47 @@ provided for clarity and simplicity.
 
 =back
 
+=head1 DATES AND TIMES
+
+The XML-RPC specification refers to the date/time values as ISO 8601, but
+unfortunately got the syntax slightly wrong in the examples. However, since
+this is the published specification it is necessary to produce time-stamps that
+conform to this format. The specification implies that the only format for
+date/time values is:
+
+    YYYYMMDDThh:mm:ss
+
+(Here, the C<T> is literal, the rest represent elements of the date and time.)
+However, the ISO 8601 specification does not allow this particular format, and
+in generally is I<considerably> more flexible than this.  Yet there are
+implementations of the XML-RPC standard in other languages that rely on a
+strict interpretation of this format.
+
+To accommodate this, the B<RPC::XML> package only produces B<dateTime.iso8601>
+values in the format given in the spec, with the possible addition of timezone
+information if the string used to create a B<RPC::XML::datetime_iso8601>
+instance included a timezone offset. The string passed in to the constructor
+for that class must match:
+
+    \d\d\d\d-?\d\d-?\d\dT?\d\d:\d\d:\d\d([.,]\d+)?(Z|[-+]\d\d:\d\d)?
+
+This pattern is also used by B<smart_encode> to distinguish a date/time string
+from a regular string. Note that the C<T> is optional here, as it is in the
+ISO 8601 spec. The timezone is optional, and if it is not given then UTC is
+assumed. The XML-RPC specification says not to assume anything about the
+timezone in the absence of one, but the format of ISO 8601 declares that that
+absence of an explicit timezone dictates UTC.
+
+If you have L<DateTime::Format::ISO8601|DateTime::Format::ISO8601> installed,
+then B<RPC::XML::datetime_iso8601> will fall back on it to try and parse any
+input strings that do not match the above pattern. If the string cannot be
+parsed by the B<DateTime::Format::ISO8601> module, then the constructor returns
+B<undef> and B<$RPC::XML::ERROR> is set.
+
 =head1 DIAGNOSTICS
 
 All constructors (in all data classes) return C<undef> upon failure, with the
-error message available in the package-global variable B<C<$RPC::XML::ERROR>>.
+error message available in the package-global variable B<$RPC::XML::ERROR>.
 
 =head1 GLOBAL VARIABLES
 
@@ -1984,6 +2082,10 @@ L<http://cpanratings.perl.org/d/RPC-XML>
 
 L<http://search.cpan.org/dist/RPC-XML>
 
+=item * MetaCPAN
+
+L<https://metacpan.org/release/RPC-XML>
+
 =item * Source code on GitHub
 
 L<http://github.com/rjray/rpc-xml>
@@ -1992,7 +2094,7 @@ L<http://github.com/rjray/rpc-xml>
 
 =head1 LICENSE AND COPYRIGHT
 
-This file and the code within are copyright (c) 2010 by Randy J. Ray.
+This file and the code within are copyright (c) 2011 by Randy J. Ray.
 
 Copying and distribution are permitted under the terms of the Artistic
 License 2.0 (L<http://www.opensource.org/licenses/artistic-license-2.0.php>) or
@@ -2006,7 +2108,7 @@ specification.
 
 =head1 SEE ALSO
 
-L<RPC::XML::Client>, L<RPC::XML::Server>, L<RPC::XML::Parser>, L<XML::Parser>
+L<RPC::XML::Client|RPC::XML::Client>, L<RPC::XML::Server|RPC::XML::Server>
 
 =head1 AUTHOR
 
